@@ -8,11 +8,12 @@ from abc import ABC, abstractmethod, abstractproperty
 from contextlib import suppress
 
 import dask
+from dask.utils import parse_timedelta
 
 from ..metrics import time
 from ..protocol import pickle
 from ..protocol.compression import get_default_compression
-from ..utils import TimeoutError, parse_timedelta
+from ..utils import TimeoutError
 from . import registry
 from .addressing import parse_address
 
@@ -156,9 +157,9 @@ class Comm(ABC):
     def __repr__(self):
         clsname = self.__class__.__name__
         if self.closed():
-            return "<closed %s>" % (clsname,)
+            return f"<closed {clsname}>"
         else:
-            return "<%s %s local=%s remote=%s>" % (
+            return "<{} {} local={} remote={}>".format(
                 clsname,
                 self.name or "",
                 self.local_address,
@@ -306,7 +307,7 @@ async def connect(
             )
             await asyncio.sleep(backoff)
     else:
-        raise IOError(
+        raise OSError(
             f"Timed out trying to connect to {addr} after {timeout} s"
         ) from active_exception
 
@@ -322,7 +323,7 @@ async def connect(
     except Exception as exc:
         with suppress(Exception):
             await comm.close()
-        raise IOError(
+        raise OSError(
             f"Timed out during handshake while connecting to {addr} after {timeout} s"
         ) from exc
 
